@@ -29,7 +29,31 @@
 
 #ifdef USE_WIN32
 
-#define VERSION "3.6"
+/* default certificate */
+#define DEFAULT_CERT "stunnel.pem"
+
+/* additional directory (hashed!) with trusted CA client certs */
+#define CA_DIR "trusted"
+
+/* certificate used for sign our client certs */
+#define CLIENT_CA "cacert.pem"
+
+#else /* USE_WIN32 */
+
+/* default certificate */
+#define DEFAULT_CERT certdir "/stunnel.pem"
+
+/* additional directory (hashed!) with trusted CA client certs */
+#define CA_DIR certdir "/trusted"
+
+/* certificate used for signing our client certs */
+#define CLIENT_CA ssldir "/localCA/cacert.pem"
+
+#endif /* USE_WIN32 */
+
+#ifdef USE_WIN32
+
+#define VERSION "3.8"
 #define HOST "i586-pc-mingw32-gnu"
 
 #define HAVE_VSNPRINTF
@@ -52,8 +76,6 @@ int _vsnprintf(char *, int, char *, ...);
 
 #else /* USE_WIN32 */
 
-#include "config.h"
-
 #define get_last_socket_error() errno
 #define get_last_error()        errno
 #define readsocket(s,b,n)       read((s),(b),(n))
@@ -69,11 +91,6 @@ int _vsnprintf(char *, int, char *, ...);
 #define USE_FORK
 #endif
 
-/* Pseudoterminals */
-#if HAVE_PTY_H && HAVE_LIBUTIL
-#define USE_PTY
-#endif
-
 /* TCP wrapper */
 #if HAVE_TCPD_H && HAVE_LIBWRAP
 #define USE_LIBWRAP
@@ -83,6 +100,14 @@ int _vsnprintf(char *, int, char *, ...);
 
 #endif /* USE_WIN32 */
 
+#if defined(sun) && !defined(__svr4__)  /* ie. SunOS 4 */
+#define atexit(a) on_exit((a), NULL)
+extern int sys_nerr;
+extern char *sys_errlist[];
+#define strerror(num) ((num)==0 ? "No error" : \
+    ((num)>=sys_nerr ? "Unknown error" : sys_errlist[num]))
+#endif /* SunOS 4 */
+
 #ifdef USE_PTHREAD
 #define STUNNEL_TMP "stunnel " VERSION " on " HOST " PTHREAD"
 #endif
@@ -101,6 +126,12 @@ int _vsnprintf(char *, int, char *, ...);
 /* Length of strings (including the terminating '\0' character) */
 #define STRLEN         1024
 
+/* Safe copy for strings declarated as char[STRLEN] */
+#define safecopy(dst, src) \
+    (dst[STRLEN-1]='\0', strncpy((dst), (src), STRLEN-1))
+#define safeconcat(dst, src) \
+    (dst[STRLEN-1]='\0', strncat((dst), (src), STRLEN-strlen(dst)-1))
+
 typedef struct {
     char certfile[STRLEN];  /* name of the certificate */
     char clientdir[STRLEN];
@@ -120,6 +151,8 @@ typedef struct {
     char *cipher_list;
     char *username;
     char *protocol;
+    char *setuid_user;
+    char *setgid_group;
 } server_options;
 
 /* Prototypes for stunnel.c */
@@ -151,5 +184,12 @@ unsigned long process_id(void);
 unsigned long thread_id(void);
 int create_client(int, int, void (*)(int));
 
+/* Prototypes for pty.c */
+/* Based on Public Domain code by Tatu Ylonen <ylo@cs.hut.fi> */
+
+int pty_allocate(int *ptyfd, int *ttyfd, char *ttyname, int ttynamelen);
+void pty_release(char *ttyname);
+void pty_make_controlling_tty(int *ttyfd, char *ttyname);
+
 /* End of common.h */
 
