@@ -44,33 +44,6 @@
 /* I/O buffer size */
 #define BUFFSIZE       8192
 
-#ifdef USE_WIN32
-
-/* default certificate */
-#define DEFAULT_CERT "stunnel.pem"
-
-/* additional directory (hashed!) with trusted CA client certs */
-#define CA_DIR "trusted"
-
-/* certificate used for sign our client certs */
-#define CLIENT_CA "cacert.pem"
-
-#else /* USE_WIN32 */
-
-/* directory for certificate */
-#define CERT_DIR sslcnf "/certs"
-
-/* default certificate */
-#define DEFAULT_CERT CERT_DIR "/stunnel.pem"
-
-/* additional directory (hashed!) with trusted CA client certs */
-#define CA_DIR CERT_DIR "/trusted"
-
-/* certificate used for signing our client certs */
-#define CLIENT_CA sslcnf "/localCA/cacert.pem"
-
-#endif /* USE_WIN32 */
-
 #include "common.h"
 
 #include <stdio.h>
@@ -100,6 +73,9 @@
 
 #else /* defined USE_WIN32 */
 
+/* Must be included before sys/stat.h for Ultrix */
+#include <sys/types.h>   /* u_short, u_long */
+
 /* General headers */
 #include <errno.h>       /* errno */
 #include <sys/stat.h>    /* stat */
@@ -117,7 +93,6 @@
 #endif
 
 /* Networking headers */
-#include <sys/types.h>   /* u_short, u_long */
 #include <netinet/in.h>  /* struct sockaddr_in */
 #include <sys/socket.h>  /* getpeername */
 #include <arpa/inet.h>   /* inet_ntoa */
@@ -230,12 +205,13 @@ dh_done:
     SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_BOTH);
     SSL_CTX_set_timeout(ctx, options.session_timeout);
     if(options.option&OPT_CERT) {
-        log(LOG_DEBUG, "Certificate: %s", options.certfile);
         if(!SSL_CTX_use_certificate_file(ctx, options.certfile,
                 SSL_FILETYPE_PEM)) {
+            log(LOG_ERR, "Error reading certificate file: %s", options.certfile);
             sslerror("SSL_CTX_use_certificate_file");
             exit(1);
         }
+        log(LOG_DEBUG, "Certificate: %s", options.certfile);
 #ifdef NO_RSA
         if(!SSL_CTX_use_PrivateKey_file(ctx, options.certfile,
                 SSL_FILETYPE_PEM)) {
@@ -547,6 +523,7 @@ done:
 #ifndef NO_RSA
 static RSA *tmp_rsa_cb(SSL *s, int export, int keylength)
 { /* temporary RSA key callback */
+    log(LOG_DEBUG, "Returned temporary RSA callback");
     return rsa_tmp;
 }
 #endif /* NO_RSA */
@@ -644,5 +621,5 @@ static void sslerror(char *txt) /* SSL Error handler */
     log(LOG_ERR, "%s: %s", txt, string);
 }
 
-/* End of stunnel.c */
+/* End of ssl.c */
 
