@@ -3,8 +3,8 @@
  *   Copyright (c) 1998-2000 Michal Trojnara <Michal.Trojnara@centertel.pl>
  *                 All Rights Reserved
  *
- *   Version:      3.6              (stunnel.c)
- *   Date:         2000.02.03
+ *   Version:      3.8              (stunnel.c)
+ *   Date:         2000.02.24
  *   Author:       Michal Trojnara  <Michal.Trojnara@centertel.pl>
  *   SSL support:  Adam Hernik      <adas@infocentrum.com>
  *                 Pawel Krawczyk   <kravietz@ceti.com.pl>
@@ -31,29 +31,11 @@
 /* Max number of children */
 #define MAX_CLIENTS    100
 
-#ifdef USE_WIN32
-
-/* default certificate */
-#define DEFAULT_CERT "stunnel.pem"
-
-/* additional directory (hashed!) with trusted CA client certs */
-#define CA_DIR "trusted"
-
-#else /* USE_WIN32 */
-
-/* directory for certificate */
-#define CERT_DIR sslcnf "/certs"
-
-/* default certificate */
-#define DEFAULT_CERT CERT_DIR "/stunnel.pem"
-
-/* additional directory (hashed!) with trusted CA client certs */
-#define CA_DIR CERT_DIR "/trusted"
-
-#endif /* USE_WIN32 */
-
 #include "common.h"
 
+    /* Must be included before sys/stat.h for Ultrix */
+#include <sys/types.h>   /* u_short, u_long */
+
     /* General headers */
 #include <stdio.h>
 #include <errno.h>       /* errno */
@@ -81,13 +63,16 @@ static struct WSAData wsa_state;
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>      /* getpid, fork, execvp, exit */
 #endif
-#ifdef HAVE_PTY_H
-#include <pty.h>
+#ifdef HAVE_STROPTS_H
+#include <stropts.h>
 #endif
+
+#include <pwd.h>
+#include <grp.h>
+
 #include <fcntl.h>
 
     /* Networking headers */
-#include <sys/types.h>   /* u_short, u_long */
 #include <netinet/in.h>  /* struct sockaddr_in */
 #include <sys/socket.h>  /* getpeername */
 #include <arpa/inet.h>   /* inet_ntoa */
@@ -141,13 +126,6 @@ static void print_help();
 
 server_options options;
 
-    /* Macros */
-/* Safe copy for strings declarated as char[STRLEN] */
-#define safecopy(dst, src) \
-    (dst[STRLEN-1]='\0', strncpy((dst), (src), STRLEN-1))
-#define safeconcat(dst, src) \
-    (dst[STRLEN-1]='\0', strncat((dst), (src), STRLEN-strlen(dst)-1))
-
     /* Functions */
 int main(int argc, char* argv[])
 { /* execution begins here 8-) */
@@ -162,6 +140,7 @@ int main(int argc, char* argv[])
     signal(SIGPIPE, SIG_IGN); /* avoid 'broken pipe' signal */
     signal(SIGTERM, signal_handler);
     signal(SIGQUIT, signal_handler);
+    signal(SIGINT, signal_handler);
     /* signal(SIGSEGV, signal_handler); */
 #endif
 
@@ -235,8 +214,10 @@ static void get_options(int argc, char *argv[]) {
     options.cipher_list=NULL;
     options.username=NULL;
     options.protocol=NULL;
+    options.setuid_user=NULL;
+    options.setgid_group=NULL;
     opterr=0;
-    while ((c = getopt(argc, argv, "a:cp:v:d:fTl:L:r:t:u:n:hC:D:V")) != EOF)
+    while ((c = getopt(argc, argv, "a:cp:v:d:fTl:L:r:s:g:t:u:n:hC:D:V")) != EOF)
         switch (c) {
             case 'a':
                 safecopy(options.clientdir, optarg);
@@ -283,10 +264,8 @@ static void get_options(int argc, char *argv[]) {
             case 'T':
                 options.option|=OPT_TRANSPARENT;
                 break;
-#ifdef USE_PTY
             case 'L':
                 options.option |= OPT_PTY;
-#endif
             case 'l':
                 options.option |= OPT_PROGRAM;
                 options.execname = optarg;
@@ -311,6 +290,12 @@ static void get_options(int argc, char *argv[]) {
                     options.remotenames[0] = htonl(INADDR_LOOPBACK);
                 }
                 break;
+            case 's':
+                options.setuid_user=optarg;
+                break;
+            case 'g':
+                options.setgid_group=optarg;
+                break;
             case 't':
                 if(!(options.session_timeout=atoi(optarg))) {
                     log(LOG_ERR, "Illegal session timeout: %s", optarg);
@@ -337,11 +322,11 @@ static void get_options(int argc, char *argv[]) {
                 fprintf(stderr, "\n" STUNNEL_INFO "\n\n");
                 exit(0);
             case '?':
-                log(LOG_ERR, "Illegal option '%c'", optopt);
+                log(LOG_ERR, "Illegal option: '%c'", optopt);
             case 'h':
                 print_help();
             default:
-                log(LOG_ERR, "Internal error in get_options");
+                log(LOG_ERR, "INTERNAL ERROR: Illegal option: '%c'", c);
                 print_help();
         }
     if (options.option & OPT_CLIENT) {
@@ -448,6 +433,7 @@ static void daemonize() /* go to background */
 static void create_pid()
 {
     FILE *pf;
+    int oldumask;
 
     options.dpid=(unsigned long)getpid();
 #ifdef HAVE_SNPRINTF
@@ -455,9 +441,10 @@ static void create_pid()
 #else
     sprintf(options.pidfile,
 #endif
-        "/var/run/stunnel.%s.pid", options.servname);
-    umask(022);
+        "%s/stunnel.%s.pid", piddir, options.servname);
+    oldumask=umask(022);
     pf=fopen(options.pidfile, "w");
+    umask(oldumask);
     if(!pf) {
         ioerror(options.pidfile);
         return; /* not critical */
@@ -503,6 +490,45 @@ static int listen_local() /* bind and listen on local interface */
         sockerror("listen");
         exit(1);
     }
+
+#ifndef USE_WIN32
+    if(options.setgid_group) {
+        struct group *gr;
+        gid_t gr_list[1];
+
+        gr=getgrnam(options.setgid_group);
+        if(!gr) {
+            log(LOG_ERR, "Failed to get GID for group %s",
+                options.setgid_group);
+            exit(1);
+        }
+        if(setgid(gr->gr_gid)) {
+            sockerror("setgid");
+            exit(1);
+        }
+        gr_list[0]=gr->gr_gid;
+        if(setgroups(1, gr_list)) {
+            sockerror("setgroups");
+            exit(1);
+        }
+    }
+
+    if(options.setuid_user) {
+        struct passwd *pw;
+
+        pw=getpwnam(options.setuid_user);
+        if(!pw) {
+            log(LOG_ERR, "Failed to get UID for user %s",
+                options.setuid_user);
+            exit(1);
+        }
+        if(setuid(pw->pw_uid)) {
+            sockerror("setuid");
+            exit(1);
+        }
+    }
+#endif /* USE_WIN32 */
+
     return ls;
 }
 
@@ -512,54 +538,48 @@ int connect_local(u_long ip) /* connect to local host */
     log(LOG_ERR, "LOCAL MODE NOT SUPPORTED ON WIN32 PLATFORM");
     return -1;
 #else
-    {
-        struct in_addr addr;
-        char text[STRLEN];
-        int fd[2];
+    struct in_addr addr;
+    char text[STRLEN];
+    int fd[2];
 
-#ifdef USE_PTY
+    if (options.option & OPT_PTY) {
         char tty[STRLEN];
 
-        if (options.option & OPT_PTY) {
-            if(openpty(fd, fd+1, tty, NULL, NULL)<0) {
-                ioerror("openpty");
-                return -1;
-            }
-            log(LOG_DEBUG, "%s allocated", tty);
-        } else
-#endif /* USE_PTY */
-        {
-            if(make_sockets(fd))
-                return -1;
-        }
-        switch (fork()) {
-        case -1:    /* error */
-            closesocket(fd[0]);
-            closesocket(fd[1]);
-            ioerror("fork");
+        if(pty_allocate(fd, fd+1, tty, STRLEN)) {
             return -1;
-        case  0:    /* child */
-            closesocket(fd[0]);
-            dup2(fd[1], 0);
-            dup2(fd[1], 1);
-            if (!options.foreground)
-                dup2(fd[1], 2);
-            closesocket(fd[1]);
-            if (ip) {
-                putenv("LD_PRELOAD=" libdir "/stunnel.so");
-                addr.s_addr = ip;
-                safecopy(text, "REMOTE_HOST=");
-                safeconcat(text, inet_ntoa(addr));
-                putenv(text);
-            }
-            execvp(options.execname, options.execargs);
-            ioerror("execvp"); /* execv failed */
-            exit(1);
         }
-        /* parent */
-        closesocket(fd[1]);
-        return fd[0];
+        log(LOG_DEBUG, "%s allocated", tty);
+    } else {
+        if(make_sockets(fd))
+            return -1;
     }
+    switch (fork()) {
+    case -1:    /* error */
+        closesocket(fd[0]);
+        closesocket(fd[1]);
+        ioerror("fork");
+        return -1;
+    case  0:    /* child */
+        closesocket(fd[0]);
+        dup2(fd[1], 0);
+        dup2(fd[1], 1);
+        if (!options.foreground)
+            dup2(fd[1], 2);
+        closesocket(fd[1]);
+        if (ip) {
+            putenv("LD_PRELOAD=" libdir "/stunnel.so");
+            addr.s_addr = ip;
+            safecopy(text, "REMOTE_HOST=");
+            safeconcat(text, inet_ntoa(addr));
+            putenv(text);
+        }
+        execvp(options.execname, options.execargs);
+        ioerror("execvp"); /* execv failed */
+        exit(1);
+    }
+    /* parent */
+    closesocket(fd[1]);
+    return fd[0];
 #endif /* USE_WIN32 */
 }
 
@@ -894,6 +914,8 @@ static void print_help()
         "\n\t\tdefault: background in daemon mode"
         "\n  -l program\texecute local inetd-type program"
         "\n  -L program\topen local pty and execute program"
+        "\n  -s username\tsetuid() to username in daemon mode"
+        "\n  -g groupname\tsetgid() to groupname in daemon mode"
 #endif
         "\n  -r [ip:]port\tconnect to remote service"
         " (ip defaults to INADDR_LOOPBACK)"
